import { GetPlacesResponse } from '~/business-logic/core/places/get-places/data/response-object/protocols';
import IGetPlacesPort from '~/business-logic/core/places/get-places/port';
import { HTTPPovider } from '~/driven/boundaries/http-boundary/httpBoundary';
import { HttpOptionsType } from '~/driven/boundaries/http-boundary/protocols';
import { apiUrls } from '~/driven/utils/configs/appConfig';
import AdminUserModel from '~/business-logic/generic/admin-user/common/data/model/adminUserModel';
import { getPlacesAdapterReturnType } from './protocols';

const getPlacesAdapter = (
  userAdmin: AdminUserModel,
  updateAccessToken: (newAccessToken: string) => void,
  navigateToAuth: () => void,
): IGetPlacesPort & getPlacesAdapterReturnType => {
  // url of api
  const url = apiUrls.core.getPlaces;
  // make the options of request
  const options: HttpOptionsType = {
    url,
    method: 'GET',
  };
  // make the httpHandler
  const httpProvider = new HTTPPovider(
    {
      accessToken: userAdmin.adminUserData.accessToken,
      refreshToken: userAdmin.adminUserData.refreshToken,
    },
    updateAccessToken,
    navigateToAuth,
  );

  const httpHandler = async () => httpProvider.request<GetPlacesResponse>(options);

  // return the method
  return {
    httpHandler,
    url,
  };
};

export default getPlacesAdapter;
