import { apiUrls } from '~/driven/utils/configs/appConfig';
import { HttpOptionsType } from '~/driven/boundaries/http-boundary/protocols';
import { HTTPPovider } from '~/driven/boundaries/http-boundary/httpBoundary';
import { GetUsersResponse } from '~/business-logic/core/users/get-users/data/response-object/protocols';
import IGetUsersPort from '~/business-logic/core/users/get-users/ports';
import AdminUserModel from '~/business-logic/generic/admin-user/common/data/model/adminUserModel';
import { getUsersAdapterReturnType } from './protocols';

const getUsersAdapter = (
  userAdmin: AdminUserModel,
  updateAccessToken: (newAccessToken: string) => void,
  navigateToAuth: () => void,
): IGetUsersPort & getUsersAdapterReturnType => {
  // url of api
  const url = apiUrls.core.getUsers;
  // make the options of request
  const options: HttpOptionsType = {
    url,
    method: 'GET',
  };
  // make the httpHandler
  const httpProvider = new HTTPPovider(
    {
      accessToken: userAdmin.adminUserData.accessToken,
      refreshToken: userAdmin.adminUserData.refreshToken,
    },
    updateAccessToken,
    navigateToAuth,
  );

  const httpHandler = async () => httpProvider.request<GetUsersResponse>(options);

  // return the method
  return {
    httpHandler,
    url,
  };
};

export default getUsersAdapter;
