import React from 'react';
import ReactDOM from 'react-dom';

export interface notifInterface {
  type: 'error' | 'success' | 'warning';
  message: string;
  time?: number;
  onCloseCallback?: () => unknown;
}
export default function Notification({ message, type, time = 5, onCloseCallback }: notifInterface) {
  const notifRef = React.useRef<HTMLDivElement>(null);
  const el = React.useRef(document.createElement('div'));

  React.useEffect(() => {
    const portal = document.getElementById('root');
    portal?.appendChild(el.current);

    setTimeout(() => {
      el.current.remove();
      if (typeof onCloseCallback !== 'undefined') onCloseCallback();
    }, 1000 * time);

    return () => {
      if (typeof onCloseCallback !== 'undefined') onCloseCallback();
      return el.current?.remove();
    };
  }, []);

  return ReactDOM.createPortal(
    <div
      ref={notifRef}
      className={`fixed top-10 left-1/2 translate-x-[-50%] z-30 p-2 rounded-md text-black ${
        type === 'error' && 'bg-red-600 text-white'
      } ${type === 'success' && 'bg-green-600 text-white'} ${type === 'warning' && 'bg-yellow-500'}`}
    >
      {message}
    </div>,
    el.current,
  );
}
