import StateManagementService from '~/driven/boundaries/state-management';
import StorageService from '~/driven/boundaries/storage-boundary';
import { NavigateFunction } from 'react-router-dom';
import { appConfig, routes } from '../configs/appConfig';
import { errorHandlingStateTypes, UIErrorHandling } from './protocols/globalHelpersProtocols';
import { IUserContext } from './contexts/userContext';

export const UIErrorHandlingFactory = <DATA_RESPONSE>({
  state,
  message,
  data,
}: {
  state: `${errorHandlingStateTypes}`;
  message: string;
  data: DATA_RESPONSE;
}): UIErrorHandling<DATA_RESPONSE> => ({
  data,
  message,
  state,
});

export const prepareStateManagementForVM = <ReturnType>(apiUrl: string, model: () => Promise<ReturnType>) => {
  const stateManagement = StateManagementService.swr();

  const useGetPlacesList = () => stateManagement.useGetQuery(apiUrl, model);

  return useGetPlacesList;
};

export const checkPhoneNumberInput = (newValue: string) => {
  return (Number.isFinite(+newValue) || newValue === '+') && newValue.length <= 12;
};

export const updateAccessToken = (newAccessToken: string, userContext: IUserContext) => {
  const { setUser, user } = userContext;
  if (!user || !setUser) return;
  const storage = StorageService.localStorage();
  user.adminUserData.accessToken = newAccessToken;
  storage.setData(appConfig.adminUserStorageKey, user);
  setUser(user);
};

export const navigateToAuth = (userCtx: IUserContext, navigate: NavigateFunction) => {
  const { setUser } = userCtx;
  const storage = StorageService.localStorage();
  storage.deleteData(appConfig.adminUserStorageKey);
  if (!setUser) return;
  setUser(null);
  navigate(routes.authentication);
};
