import { useState } from 'react';
import { checkPhoneNumberInput } from '~/driven/utils/helpers/globalHelpers';
import { INewUserData } from '~/business-logic/core/users/create-user/create-account/data/dto/protocols';
import { AxiosError } from 'axios';
import { staticMessages } from '~/driven/utils/constants/staticMessages';
import ICreateUserViewProps from '../view/protocols';
import IUseCreateUserVm from './protocols';

const inputStateInitialValue: INewUserData = {
  firstname: '',
  lastname: '',
  phonenumber: '',
};

const inputNames: (keyof INewUserData)[] = ['firstname', 'lastname', 'phonenumber'];

const useCreateUserVM = (dependencies: IUseCreateUserVm): ICreateUserViewProps => {
  const [error, setError] = useState<{ message: string; type: 'error' | 'success' }>({ message: '', type: 'error' });

  const { handleSubmitForm } = dependencies;
  const [inputsValue, setInputValues] = useState<INewUserData>(inputStateInitialValue);

  const inputsSetStates = (name: keyof INewUserData, newValue: string) => {
    if (name === 'phonenumber' && !checkPhoneNumberInput(newValue)) return;
    setInputValues((prev) => ({
      ...prev,
      [name]: newValue,
    }));
  };

  const onSubmitCreateUserForm = async (e: React.FormEvent) => {
    e.preventDefault();
    try {
      await handleSubmitForm(inputsValue);
      setError({ message: staticMessages.global.success.createUser, type: 'success' });
    } catch (errorExc) {
      console.log('herere', errorExc);
      if (errorExc instanceof AxiosError) {
        console.log('herere', errorExc);
        setError({ message: errorExc.response?.data?.description, type: 'error' });
      } else if (errorExc instanceof Error) {
        setError({ message: errorExc.message, type: 'error' });
      }
    }
  };

  const inputStates: INewUserData = { ...inputsValue };

  return {
    stateHandler: {
      inputsSetStates,
      inputStates,
    },
    onSubmit: onSubmitCreateUserForm,
    inputNames,
    error,
    setError,
  };
};

export default useCreateUserVM;
