import React from 'react';
import getPlaces from '~/business-logic/core/places/get-places';
import getPlacesAdapter from '~/driven/adapters/get-places-adapter/getPlacesAdapter';
import PlacesModel from '~/business-logic/core/places/common/model/placesModel';
import { prepareStateManagementForVM } from '~/driven/utils/helpers/globalHelpers';
import useGetNavigatorAndTokenUpdater from '~/driven/utils/helpers/hooks/getNavigatorAndAccessTokenUpdator';
import AdminUserModel from '~/business-logic/generic/admin-user/common/data/model/adminUserModel';
import PlacesListView from '../view/PlacesListView';
import usePlacesListVM from '../viewmodel/placesListVM';
import placesListModel from '../model/placesListModel';

const prepareTheLogicForModel = () => {
  const { accessTokenUpdateHandler, notLoginAuth, userData } = useGetNavigatorAndTokenUpdater();

  const { user } = userData;
  const gettingPlacesDrivenAdapter = getPlacesAdapter(user as AdminUserModel, accessTokenUpdateHandler, notLoginAuth);
  const { url } = gettingPlacesDrivenAdapter;
  const getingPlacesLogic = getPlaces(gettingPlacesDrivenAdapter);
  return { getingPlacesLogic, url };
};

export default function PlacessList() {
  const { getingPlacesLogic, url } = prepareTheLogicForModel();
  const placesModel = async () => await placesListModel(getingPlacesLogic);

  const useGetPlacesList = prepareStateManagementForVM<PlacesModel>(url, placesModel);
  const { selectedRowId, setSelectedRowId, placesData } = usePlacesListVM({
    useGetPlacesList,
  });
  return <PlacesListView placesList={placesData} selectedRowId={selectedRowId} setSelectedRowId={setSelectedRowId} />;
}
